<?php
class JonoUpdater
{
    private $slug;
    private $pluginData;
    private $pluginFile;
    private $APIResult;

    function __construct($pluginFile)
    {
        add_filter("pre_set_site_transient_update_plugins", [$this, "setTransitent"]);
        add_filter("plugins_api", [$this, "setPluginInfo"], 10, 3);
        add_filter("upgrader_post_install", [$this, "postInstall"], 10, 3);

        $this->pluginFile = $pluginFile;
    }
    
    private function initPluginData()
    {
        $this->slug = plugin_basename($this->pluginFile);
        $this->pluginData = get_plugin_data($this->pluginFile);
    }
    
    private function getReleaseInfo()
    {
        if (!empty($this->APIResult)) {
            return;
        }

        $url = "https://www.jono.be/api/plugin/wordpress";

        $this->APIResult = wp_remote_retrieve_body(wp_remote_get($url));
        if (!empty($this->APIResult)) {
            $this->APIResult = @json_decode($this->APIResult);
        }
        
        if (is_array($this->APIResult) && count($this->APIResult) > 0) {
            $this->APIResult = $this->APIResult[0];
        }
    }

    public function setTransitent($transient)
    {
        if (empty($transient->checked)) {
            return $transient;
        }

        $this->initPluginData();
        $this->getReleaseInfo();
        
        if (isset($transient->checked[$this->slug])) {
            $doUpdate = version_compare(
                $this->APIResult->latest_version,
                $transient->checked[$this->slug]
            );
            
            if ($doUpdate == 1) {
                $package = $this->APIResult->download_url;

                $obj = new stdClass();
                $obj->slug = $this->slug;
                $obj->new_version = $this->APIResult->latest_version;
                $obj->url = $this->pluginData["PluginURI"];
                $obj->package = $package;
                $transient->response[$this->slug] = $obj;
            }
        }
        return $transient;
    }

    public function setPluginInfo($false, $action, $response)
    {
        $this->initPluginData();
        $this->getReleaseInfo();
        
        if (empty($response->slug) || $response->slug != $this->slug) {
            return false;
        }

        $response->last_updated = $this->APIResult->updated_at;
        $response->slug = $this->slug;
        $response->plugin_name = $this->pluginData["Name"];
        $response->version = $this->APIResult->latest_version;
        $response->author = $this->pluginData["AuthorName"];
        $response->homepage = $this->pluginData["PluginURI"];

        $downloadLink = $this->APIResult->download_url;
        $response->download_link = $downloadLink;

        $response->sections = [
            "description" => $this->pluginData["Description"],
            "changelog" => $this->APIResult->changelog,
        ];

        $matches = null;
        preg_match(
            "/requires:\s([\d\.]+)/i",
            $this->APIResult->changelog,
            $matches
        );
        if (!empty($matches)) {
            if (is_array($matches)) {
                if (count($matches) > 1) {
                    $response->requires = $matches[1];
                }
            }
        }

        $matches = null;
        preg_match(
            "/tested:\s([\d\.]+)/i",
            $this->APIResult->changelog,
            $matches
        );
        if (!empty($matches)) {
            if (is_array($matches)) {
                if (count($matches) > 1) {
                    $response->tested = $matches[1];
                }
            }
        }
        return $response;
    }

    public function postInstall($true, $hook_extra, $result)
    {
        $this->initPluginData();

        if (
            !isset($hook_extra["plugin"]) ||
            $this->slug !== $hook_extra["plugin"]
        ) {
            return $result;
        }

        $wasActivated = is_plugin_active($this->slug);
        global $wp_filesystem;
        $pluginFolder = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . dirname($this->slug);
        $wp_filesystem->move($result["destination"], $pluginFolder);
        $result["destination"] = $pluginFolder;
        // Re-activate plugin if needed
        if ($wasActivated) {
            $activate = activate_plugin($this->slug);
        }
        return $result;
    }
}
