<?php
/**
 * Plugin Name: Jono AI
 * Plugin URI: https://www.jono.be
 * Description: Generate content
 * Author: Made I.T.
 * Author URI: https://www.madeit.be
 * Version: 0.2.1
 * Text Domain: jono
 * Domain Path: /languages
 * License: GPLv2.
 */

require __DIR__ . '/JonoUpdater.php';
new JonoUpdater(__FILE__);

function jono_load_plugin_textdomain()
{
    load_plugin_textdomain('jono', false, basename(dirname(__FILE__)).'/languages/');
}
add_action('plugins_loaded', 'jono_load_plugin_textdomain');

function jono_add_settingsmenu()
{
    add_menu_page('Jono', 'Jono', 'administrator', 'jono_settings', 'jono_settings', 'dashicons-admin-settings', 26);
}
add_action('admin_menu', 'jono_add_settingsmenu', 9);

function jono_settings()
{
    ?>
    <div class="wrap">
        <div id="icon-themes" class="icon32"></div>  
        <h2>Jono Instellingen</h2>
        <?php settings_errors(); ?>  
        <form method="POST" action="options.php">  
            <?php 
            settings_fields( 'jono_general_settings' );
            do_settings_sections( 'jono_general_settings' ); 
            ?>             
            <?php submit_button(); ?>  
        </form> 
    </div>
    <?php
}

function jono_settings_fields()
{
    add_settings_section('jono_general_section', '',  'jono_display_general_account', 'jono_general_settings');
    
    $args = array (
        'type' => 'input',
        'subtype' => 'text',
        'id' => 'jono_api_key',
        'name' => 'jono_api_key',
        'required' => 'true',
        'get_options_list' => '',
        'value_type'=>'normal',
        'wp_data' => 'option'
    );
    add_settings_field('jono_api_key', 'API Key', 'jono_render_settings_field', 'jono_general_settings', 'jono_general_section', $args);
    register_setting('jono_general_settings', 'jono_api_key');
}
add_action('admin_init', 'jono_settings_fields');

function jono_display_general_account()
{
    echo '<p>Voer hier de Jono API key in. Je kan deze vinden op <a href="https://www.jono.be/user/api-tokens" target="_blank">https://www.jono.be/user/api-tokens</a>.</p>';
}

function jono_render_settings_field($args)
{
    if($args['wp_data'] == 'option')
    {
        $wp_data_value = get_option($args['name']);
    }
    elseif($args['wp_data'] == 'post_meta')
    {
        $wp_data_value = get_post_meta($args['post_id'], $args['name'], true);
    }

    switch ($args['type'])
    {
        case 'input':
            $value = ($args['value_type'] == 'serialized') ? serialize($wp_data_value) : $wp_data_value;
            if($args['subtype'] != 'checkbox'){
                $prependStart = (isset($args['prepend_value'])) ? '<div class="input-prepend"> <span class="add-on">'.$args['prepend_value'].'</span>' : '';
                $prependEnd = (isset($args['prepend_value'])) ? '</div>' : '';
                $step = (isset($args['step'])) ? 'step="'.$args['step'].'"' : '';
                $min = (isset($args['min'])) ? 'min="'.$args['min'].'"' : '';
                $max = (isset($args['max'])) ? 'max="'.$args['max'].'"' : '';
                if(isset($args['disabled'])){
                    // hide the actual input bc if it was just a disabled input the informaiton saved in the database would be wrong - bc it would pass empty values and wipe the actual information
                    echo $prependStart.'<input type="'.$args['subtype'].'" id="'.$args['id'].'_disabled" '.$step.' '.$max.' '.$min.' name="'.$args['name'].'_disabled" size="40" disabled value="' . esc_attr($value) . '" /><input type="hidden" id="'.$args['id'].'" '.$step.' '.$max.' '.$min.' name="'.$args['name'].'" size="40" value="' . esc_attr($value) . '" />'.$prependEnd;
                } else {
                    echo $prependStart.'<input type="'.$args['subtype'].'" id="'.$args['id'].'" "'.$args['required'].'" '.$step.' '.$max.' '.$min.' name="'.$args['name'].'" size="40" value="' . esc_attr($value) . '" />'.$prependEnd;
                }
            } else {
                $checked = ($value) ? 'checked' : '';
                echo '<input type="'.$args['subtype'].'" id="'.$args['id'].'" "'.$args['required'].'" name="'.$args['name'].'" size="40" value="1" '.$checked.' />';
            }
            break;
        default:
            # code...
            break;
    }
}

function jono_add_meta_boxes()
{
    if(jono_can_do_shomething()) {
        add_meta_box('jono_woo', __('Jono AI', 'forms-by-made-it'), 'jono_woo_metabox', get_post_types(['public' => true]), 'normal', 'high');
    }
}
add_action('add_meta_boxes', 'jono_add_meta_boxes');

function jono_woo_metabox()
{
    global $post;
    ?>
    <p>
        Jono AI is de nederlandstalige content AI.
    </p>
    <?php 
    
    if(empty($post->post_title) || empty($post->post_content)) {
        ?>
        <div id="message" class="error notice"><p>Om goed te functioneren is er een titel en/of inhoud nodig.</p></div>
        <?php
    }
    
    if($post->post_type === 'product') {
        ?>
        <a href="#" id="jonoGenerateProductContent" class="button button-primary button-large">Genereer product beschrijving.</a>
        <script>
            const jonoGenerateProductContent = document.getElementById('jonoGenerateProductContent');
            jonoGenerateProductContent.addEventListener('click', function(e) {
                e.preventDefault();
                jonoGenerateProductContent.setAttribute('disabled', true);
                var data = {
                    'action': 'jono_generate_woo_content',
                    'post_id': <?php echo $post->ID; ?>
                };

                jQuery.post(ajaxurl, data, function(response) {
                    if(response.success) {
                        alert('Content gegenereerd');
                        location.reload();
                    } else {
                        alert(response.message);
                    }
                });
            });
        </script>
        <?php
    }
    
    if(!empty($post->post_content)) {
        ?>
        <a href="#" id="jonoGenerateYoastDescription" class="button button-primary button-large">Genereer Yoast SEO Meta description.</a>
        <script>
            const jonoGenerateYoastDescription = document.getElementById('jonoGenerateYoastDescription');
            jonoGenerateYoastDescription.addEventListener('click', function(e) {
                e.preventDefault();
                jonoGenerateYoastDescription.setAttribute('disabled', true);
                var data = {
                    'action': 'jono_generate_yoast_description',
                    'post_id': <?php echo $post->ID; ?>
                };

                jQuery.post(ajaxurl, data, function(response) {
                    if(response.success) {
                        alert('Content gegenereerd');
                        location.reload();
                    } else {
                        alert(response.message);
                    }
                });
            });
        </script>
        <?php
    }
    
    if(!empty($post->post_title) && empty($post->post_content) && $post->post_type === 'post') {
        ?>
        <a href="#" id="jonoGenerateBlogIdea" class="button button-primary button-large">Genereer Blog idee.</a>
        <script>
            const jonoGenerateBlogIdea = document.getElementById('jonoGenerateBlogIdea');
            jonoGenerateBlogIdea.addEventListener('click', function(e) {
                e.preventDefault();
                jonoGenerateBlogIdea.setAttribute('disabled', true);
                var data = {
                    'action': 'jono_generate_blogidea',
                    'post_id': <?php echo $post->ID; ?>
                };

                jQuery.post(ajaxurl, data, function(response) {
                    if(response.success) {
                        alert('Content gegenereerd');
                        location.reload();
                    } else {
                        alert(response.message);
                    }
                });
            });
        </script>
        <?php
    }
}

function jono_can_do_shomething()
{
    global $post;
    
    //Has Yoast SEO?
    if(in_array('wordpress-seo/wp-seo.php', apply_filters('active_plugins', get_option('active_plugins')))) {
        return true;
    }
    return false;
}

function jono_generate_woo_content()
{
    $post = get_post($_POST['post_id']);
    if($post->post_type === 'product') {
        if(!empty($post->post_content)) {
            wp_send_json(['success' => false, 'message' => 'De product inhoud is niet leeg.']);
        }
        
        list($result, $data) = jono_generate_content([
            'type' => 'App\Actions\Ai\BlogSection',
            'title' => $post->post_title,
            'description' => $post->post_title,
            'tone' => 'convincing',
        ]);
        if($result === false) {
            wp_send_json(['success' => false, 'message' => 'Kon geen contact leggen met de Jono API. Controlleer je API key.']);
        }
        else {
            if(isset($data['data']['id'])) {
                $id = $data['data']['id'];
                $i = 0;
                $res = null;
                do {
                    $isPending = true;
                    list($result, $data) = jono_get_content($id);
                    if($result === true) {
                        if(isset($data['data'])) {
                            $isPending = in_array($data['data']['status'], ['created', 'pending']);
                            
                            if($data['data']['status'] === 'finished') {
                                $res = $data['data']['contents'][0]['content'];
                                wp_update_post([
                                    'ID' => $post->ID,
                                    'post_content' => nl2br($res),
                                ]);
                            }
                        }
                    }
                    else {
                        $res = $data;
                    }
                    
                    $i++;
                    if($i > 10) {
                        $isPending = false;
                    }
                    if($isPending) {
                        sleep(3);
                    }
                } while($isPending);
                
                wp_send_json(['success' => true]);
            }
            else {
                wp_send_json(['success' => false, 'message' => json_encode($data)]);
            }
        }
        
        wp_send_json(['success' => true]);
    }
    wp_send_json(['success' => false, 'message' => 'Mislukt.']);
}
add_action('wp_ajax_jono_generate_woo_content', 'jono_generate_woo_content');

function jono_generate_yoast_description()
{
    $post = get_post($_POST['post_id']);
    $description = get_post_meta($post->ID, '_yoast_wpseo_metadesc', true);
    
    if(!empty($description)) {
        wp_send_json(['success' => false, 'message' => 'De metadescription is niet leeg.']);
    }

    list($result, $data) = jono_generate_content([
        'type' => 'App\Actions\Ai\SEODescription',
        'title' => $post->post_title,
        'tone' => 'convincing',
        'description' => substr(strip_tags($post->post_content), 0, min(250, strlen(strip_tags($post->post_content)))),
    ]);
    if($result === false) {
        wp_send_json(['success' => false, 'message' => 'Kon geen contact leggen met de Jono API. Controlleer je API key.']);
    }
    else {
        if(isset($data['data']['id'])) {
            $id = $data['data']['id'];
            error_log($id);
            $i = 0;
            $res = null;
            do {
                $isPending = true;
                list($result, $data) = jono_get_content($id);
                if($result === true) {
                    if(isset($data['data'])) {
                        error_log($data['data']['status']);
                        $isPending = in_array($data['data']['status'], ['created', 'pending']);

                        if($data['data']['status'] === 'finished') {
                            $res = $data['data']['contents'][0]['content'];
                            update_post_meta($post->ID, '_yoast_wpseo_metadesc', $res);
                        }
                    }
                }
                else {
                    $res = $data;
                }

                $i++;
                if($i > 10) {
                    $isPending = false;
                }
                if($isPending) {
                    sleep(3);
                }
            } while($isPending);

            wp_send_json(['success' => true]);
        }
        else {
            wp_send_json(['success' => false, 'message' => json_encode($data)]);
        }
    }
        
    wp_send_json(['success' => false, 'message' => 'Mislukt.']);
}
add_action('wp_ajax_jono_generate_yoast_description', 'jono_generate_yoast_description');

function jono_generate_blogidea()
{
    $post = get_post($_POST['post_id']);
    if($post->post_type === 'post') {
        if(!empty($post->post_content)) {
            wp_send_json(['success' => false, 'message' => 'De inhoud is niet leeg.']);
        }
        
        list($result, $data) = jono_generate_content([
            'type' => 'App\Actions\Ai\BlogIdea',
            'title' => $post->post_title,
            'description' => $post->post_title,
            'tone' => 'professional',
        ]);
        if($result === false) {
            wp_send_json(['success' => false, 'message' => 'Kon geen contact leggen met de Jono API. Controlleer je API key.']);
        }
        else {
            if(isset($data['data']['id'])) {
                $id = $data['data']['id'];
                $i = 0;
                $res = null;
                do {
                    $isPending = true;
                    list($result, $data) = jono_get_content($id);
                    if($result === true) {
                        if(isset($data['data'])) {
                            $isPending = in_array($data['data']['status'], ['created', 'pending']);
                            
                            if($data['data']['status'] === 'finished') {
                                $res = $data['data']['contents'][0]['content'];
                                wp_update_post([
                                    'ID' => $post->ID,
                                    'post_content' => nl2br($res),
                                ]);
                            }
                        }
                    }
                    else {
                        $res = $data;
                    }
                    
                    $i++;
                    if($i > 5) {
                        $isPending = false;
                    }
                    if($isPending) {
                        sleep(3);
                    }
                } while($isPending);
                
                wp_send_json(['success' => true]);
            }
            else {
                wp_send_json(['success' => false, 'message' => json_encode($data)]);
            }
        }
        
        wp_send_json(['success' => true]);
    }
    wp_send_json(['success' => false, 'message' => 'Mislukt.']);
}
add_action('wp_ajax_jono_generate_blogidea', 'jono_generate_blogidea');

function jono_generate_content($data)
{
    $apiKey = get_option('jono_api_key');
    
    $response = wp_remote_post('https://www.jono.be/api/1.0/content', [
        'headers' => [
            'Authorization' => 'Bearer ' . $apiKey,
        ],
        'body' => [
            'type' => $data['type'],
            'tone' => $data['tone'] ?? 'normal', //normal,friendly,luxurious,relaxed,professional,bold,Adventurous,witty,persuasive,empathetic,convincing
            'title' => $data['title'],
            'description' => $data['description'],
            'variants' => 1,
        ]
    ]);
    
    if ( is_array( $response ) && ! is_wp_error( $response ) ) {
        $headers = $response['headers'];
        $body = $response['body'];
        return [true, json_decode($body, true)];
    }
    return [false, $response->get_error_message()];
}

function jono_get_content($id)
{
    $apiKey = get_option('jono_api_key');
    
    $response = wp_remote_get('https://www.jono.be/api/1.0/content/' . $id, [
        'headers' => [
            'Authorization' => 'Bearer ' . $apiKey,
        ]
    ]);
        
    if ( is_array( $response ) && ! is_wp_error( $response ) ) {
        $headers = $response['headers'];
        $body = $response['body'];
        
        return [true, json_decode($body, true)];
    }
    return [false, $response->get_error_message()];
}

function jono_notice()
{
    if(empty(get_option('jono_api_key'))) {
        ?>
        <div id="message" class="updated notice is-dismissable"><p>Voer de jono API key in om content te genereren. <a href="/wp-admin/admin.php?page=jono_settings">Jono instellingen</a></p></div>
        <?php
    }
}
add_action('admin_notices', 'jono_notice');